package DBCapsulator;

use strict;

our( @ISA );


our $databases = {};
our $schemadir;
our $id = __PACKAGE__;


sub config {
    my( $cfg ) = @_;

    while( @$cfg ) {
        my $next = shift @$cfg;
        chomp $next;
        next if( $next =~ /^[\s\t]*#/ || $next =~ /^[\s\t]*$/ );
	my $cfg = {};
	foreach my $def ( split( /[\s\t]+/, $next ) ) {
	    if( $def =~ /^(.*?)=(.*)$/ ) {
	        $cfg->{$1} = $2;
	    }
	}
	unless( defined ($cfg->{"db"}) ) {
	    print STDERR "invalid database specifier: $next\n";
	    next;
	}
        $databases->{$cfg->{"db"}} = $cfg;
    }
}


sub schemadir {
    my( $dir ) = @_;

    $schemadir = $dir if( $dir );
    return( $schemadir );
}



sub error {
    my( $msg ) = @_;

    print STDERR "$msg\n";
    return undef;
}



sub database {
    my( $this, $db ) = @_;
    return unless( defined $databases->{$db} );
    my $dbcfg = $databases->{$db};
    return error( "no database driver: $dbcfg->{driver} ($db)" )
      unless( $dbcfg->{"driver"} );
    my $driver = $dbcfg->{"driver"};
    $driver = "DBCapsulator::$driver";
    eval "require $driver;";
    return error( "database driver $driver failed to load for $db: $@" ) if( $@ );

    my $self = $driver->connect($dbcfg);
    return $self;
}


sub connect {
    my( $this, $dbcfg ) = @_;

    my  $class = ref($this) || $this;
    my $self = {};
    bless( $self, $class );
    $self->dbcfg( $dbcfg );
    return $self;
}


sub dbcfg {
    my( $self, $dbcfg ) = @_;

    $self->{"dbcfg"} = $dbcfg if( defined $dbcfg );
    return( $self->{"dbcfg"} );
}


sub search {
    my( $self, $filter ) = @_;

    return 1;
}


sub getnext {
    my( $self ) = @_;

    return undef;
}


sub add {
    my( $self, $record ) = @_;

    return 0;
}


sub delete {
    my( $self, $record ) = @_;

    return 0;
}


sub update {
    my( $self, $record ) = @_;
    return 0;
}


sub close {
}


## @method % getnext_locallysorted(SearchFilter filter)
# Returns the next record from a search. However, records are
# first read into a local cache of the given size, then sorted
# within this cache and retrieved only after that. Records that
# are out of order after this local sort are silently dropped.
#
# @param filter the filter (only the order set in the filter is obeyed)
# @return the next record
# @sa clear_sortcache()
# @todo implement this :-)
sub getnext_locallysorted {
    my( $self, $filter ) = @_;

    unless( defined $self->{"$id::sortcache"} ) {
        $self->clear_sortcache( 1000 );
    }
    return $self->getnext();
}


## @method void clear_sortcache($size)
# Clears and setup the cache used by getnext_locallysorted(). It is good style
# to clear the cache before you first use getnext_locallysorted() on
# a started search.
#
# @param size The size of the sort cache.
# @sa getnext_locallysorted
sub clear_sortcache {
    my( $self, $size ) = @_;

    $self->{"$id::sortcache"} = [];
    $self->{"$id::sortsize"} = $size;
    $self->{"$id::sortfetch"} = 1;
}


our %schemacache;

sub schema {
    my( $self, $name ) = @_;

    $name = $self unless( defined $name );
    return( $schemacache{$name} ) if( defined $schemacache{$name} );
    my @schema;
    open( _SCHEMA, "<$schemadir/$name.dbschema" ) || return;
    while( <_SCHEMA> ) {
        chomp;
        next if( /^[\s\t]*#/ || /^[\s\t]*$/ );
	my( $ent, $type, @flags ) = split( /[\s\t]+/ );
	next unless( $ent && $type );
	my $size;
	if( $type =~ /(.*)\((\d+)\)/ ) {
	    $type = $1;
	    $size = $2;
	}
	my %flags = map { $_ => 1 } @flags;
	push( @schema, {
	    "entity" => $ent,
	    "type" => $type,
	    "size" => $size,
	    "flags" => \%flags,
	} );
    }
    $schemacache{$name} = \@schema;
    close SCHEMA;
    return \@schema;
}

1;
