package DBCapsulator::DBI;

use strict;
use DBCapsulator;
use DBI;

our( @ISA );
@ISA = qw( DBCapsulator );


sub connect {
    my( $this, $dbcfg ) = @_;

    my $self = DBCapsulator::connect( $this, $dbcfg );
    #print STDERR "connecting to table $dbcfg->{table}\n";
    my $dbh = DBI->connect( $dbcfg->{"datasource"}, $dbcfg->{"user"}, $dbcfg->{"password"}, { "AutoCommit" => 1 } );
    die "failed to connect via DBI to $dbcfg->{datasource}" unless( $dbh );
    $dbh->{"FetchHashKeyName"} = "NAME_lc";
    $self->dbh( $dbh );
    my $table = $self->dbcfg()->{"table"};
    my @names = $dbh->tables( undef, undef, $table, undef );
    my @names = grep( /^(|.*\W)$table(\W.*|)$/, @names );
    unless( @names ) {
        $self->create_table() || die "table $table does not exist in $dbcfg->{'datasource'} and create table did not succeed";
    }
    return $self;
}


sub dbh {
    my( $self, $handle ) = @_;

    $self->{"DBI::dbh"} = $handle if( defined $handle );
    return( $self->{"DBI::dbh"} );
}



sub create_table {
    my( $self ) = @_;

    my $table = $self->dbcfg()->{"table"};
    my $schema = $self->schema( $table );
    return(0) unless( $schema );
    my $sql = "CREATE TABLE $table (";
    my $pkey = "";
    for( my $i=0; $i<=$#$schema; $i++ ) {
        $sql .= "," if( $i );
	$sql .= $schema->[$i]->{"entity"};
	$sql .= " ";
	$sql .= $schema->[$i]->{"type"};
	if( $schema->[$i]->{"size"} ) {
	    $sql .= "(".($schema->[$i]->{"size"}).")";
	}
	if( $schema->[$i]->{"flags"}->{"pkey"} ) {
	    $pkey .= "," if( $pkey );
	    $pkey .= $schema->[$i]->{"entity"};
	    $sql .= " NOT NULL";
	};
    }
    $sql .= ",PRIMARY KEY ($pkey)" if( $pkey );
    $sql .= ")";
    return $self->dbh()->do( $sql );
}



sub search {
    my( $self, $filter ) = @_;

    my $sth = $self->dbh()->prepare( "SELECT * FROM ".($self->dbcfg()->{"table"}) );
    $sth->execute();
    $self->{"DBI::sth"} = $sth;
}
sub search {
    my( $self, $filter ) = @_;

    my $sth;
    if( $filter ) {
	$sth = $self->dbh()->prepare(
	    "SELECT * FROM ".
	    $self->dbcfg()->{"table"}.
	    " WHERE ".($filter->sqlexpr()).
	    " ".($filter->sqlorder()) );
    }
    else {
	$sth = $self->dbh()->prepare( "SELECT * FROM ".($self->dbcfg()->{"table"}) );
    }
    if( $sth->execute() ) {
	$self->{"DBI::search"} = $sth;
	return 1;
    }
    else {
	$self->{"DBI::search"} = undef;
	return 0;
    }
}


sub getnext {
    my( $self ) = @_;

    return undef unless( $self->{"DBI::search"} );
    my $record = $self->{"DBI::search"}->fetchrow_hashref();
    unless( $record ) {
        $self->{"DBI::search"} = undef;
    }
    return $record;
}



sub add {
    my( $self, $record ) = @_;

    my $names = "";
    my @values = ();
    my $placeholders = "";
    while( my( $col, $val ) = each %$record ) {
        if( $names ) {
	    $names .= ",";
	    $placeholders .= ",";
	}
	$names .= $col;
	$placeholders .= "?";
	push( @values, $val );
    }
    my $sth = $self->dbh()->prepare( 
        "INSERT INTO ".
	$self->dbcfg()->{"table"} .
	"($names) VALUES ($placeholders)"
    );
    $sth->execute( @values );
}


sub delete {
    my( $self, $record ) = @_;

    return 0;
}


sub update {
    my( $self, $record ) = @_;
    return 0;
}

sub close {
    my( $self ) = @_;

    $self->dbh()->disconnect();
}



1;
